declare enum TypesFn {
    fn = 0,
    arrowFn = 1,
    propFn = 2
}
declare function getHeaderFn(code: string): {
    name: string;
    args: string;
    content: string | undefined;
    isAsync: boolean;
    isStar: boolean;
    typeFn: TypesFn;
};

interface RealItem<T> {
    readonly "@hidden": boolean;
    readonly "@value": T;
}
declare namespace Data {
    interface Link {
        readonly "@t": "link";
        readonly "@type": "object" | "function";
        readonly "@link": string;
        readonly "@name": string | null;
    }
    interface String {
        readonly "@t": "string";
        "@first": boolean;
        "@value": string;
    }
    interface Number {
        readonly "@t": "number";
        readonly "@value": string;
    }
    interface BigInt {
        readonly "@t": "bint";
        readonly "@value": string;
    }
    interface Symbol {
        readonly "@t": "symbol";
        readonly "@value": string;
    }
    interface Nill {
        readonly "@t": "nill";
        readonly "@value": "null" | "undefined";
    }
    interface Function {
        readonly "@t": "function";
        readonly "@first": boolean;
        readonly "@header": ReturnType<typeof getHeaderFn>;
        readonly "@code": string;
        readonly "@real": Link | null;
    }
    interface Collection extends Omit<Record, "@t" | "@des" | "@first" | "@real"> {
        readonly "@t": "collection";
        readonly "@name": "map" | "weakmap" | "set" | "weakset";
        readonly "@size": number | null;
        readonly "@entries": unknown;
        readonly "@real": Link;
    }
    interface RegExp extends Omit<Record, "@t" | "@de" | "@real" | "@des"> {
        readonly "@t": "regexp";
        readonly "@flags": string;
        readonly "@source": string;
        readonly "@real": Link | null;
    }
    interface GetSetter {
        readonly "@t": "gs";
        readonly "@value": Link;
        readonly "@at": Partial<{
            [name in "get" | "set"]: Function;
        }>;
    }
    interface objReal {
        readonly [name: string]: RealItem<GetSetter | String | Number | BigInt | Symbol | Function | Collection | RegExp | Record | Nill | Error | Array | Element | Promise | Date | TypedArray | Buffer | DataView>;
    }
    interface Record {
        readonly "@t": "object";
        readonly "@name": string | null;
        readonly "@first": boolean;
        readonly "@real": objReal | Link;
        readonly "@des": {
            readonly "@value": DataPreview.objReal;
            readonly "@lastKey": string;
        } | null;
    }
    interface Error {
        readonly "@t": "error";
        readonly "@first": boolean;
        readonly "@stack": string;
        readonly "@real": Link | null;
    }
    interface Array extends Pick<Record, "@des"> {
        readonly "@t": "array";
        readonly "@size": number;
        readonly "@name": string | null;
        readonly "@first": boolean;
        readonly "@real": Link;
    }
    type ArrayReal = objReal & {
        readonly length: RealItem<Number>;
    };
    interface Element {
        readonly "@t": "element";
        readonly "@name": string;
        readonly "@first": boolean;
        readonly "@attrs"?: [string, string][];
        readonly "@real": Link | null;
        readonly "@childs"?: string | null | Link;
    }
    interface Promise {
        readonly "@t": "promise";
        readonly "@first": boolean;
        readonly "@state": "pending" | "fulfilled" | "rejected";
        readonly "@real": Link;
        readonly "@des": Exclude<Record["@des"], null>;
    }
    interface Date extends Pick<Record, "@des"> {
        readonly "@t": "date";
        readonly "@first": boolean;
        readonly "@value": string;
        readonly "@real": Link | null;
    }
    interface TypedArray extends Omit<Array, "@t"> {
        readonly "@t": "typedarray";
        readonly "@real": Link;
    }
    type TypedArrayReal = ArrayReal & {
        readonly buffer: RealItem<Buffer>;
        readonly byteLength: RealItem<Number>;
        readonly byteOffset: RealItem<Number>;
        readonly [Symbol.toStringTag]: RealItem<String>;
    };
    interface Buffer extends Omit<Array, "@t" | "@des" | "@real"> {
        readonly "@t": "buffer";
        readonly "@real": Link;
    }
    type BufferReal = objReal & {
        readonly byteLength: RealItem<Number>;
        readonly "[[Int8Array]]": RealItem<TypedArray>;
        readonly "[[Uint8Array]]": RealItem<TypedArray>;
        readonly "[[Int16Array]]": RealItem<TypedArray>;
        readonly "[[Int32Array]]": RealItem<TypedArray>;
        readonly "[[ArrayBufferByteLength]]": RealItem<Number>;
    };
    interface DataView extends Omit<Array, "@t" | "@des" | "@name"> {
        readonly "@t": "dataview";
        readonly "@name": "DataView";
        readonly "@real": Link;
    }
    type DataViewReal = ArrayReal & {
        readonly buffer: RealItem<Buffer>;
        readonly byteLength: RealItem<Number>;
        readonly byteOffset: RealItem<Number>;
    };
}
declare function readLinkObject(link: Data.Link): ReturnType<typeof _Encode>;
declare function callFnLink(link: Data.Link): ReturnType<typeof _Encode> | Data.Error;
declare function _getListLink(link: Data.Link): readonly (ReturnType<typeof _Encode> | Data.Error)[];
declare function clearLinkStore(): void;
declare function _Encode(data: unknown, first: boolean, // false,
linkReal: boolean, // false
preview?: boolean, forceObject?: boolean): Data.String | Data.Number | Data.BigInt | Data.Symbol | Data.Function | Data.Collection | Data.RegExp | Data.Nill | Data.Record | Data.Error | Data.Array | Data.Element | Data.Promise | Data.Date | Data.TypedArray | Data.Buffer | Data.DataView;
declare function Encode(data: unknown, deepLink?: false | number, first?: boolean, linkReal?: boolean): ReturnType<typeof _Encode> & {
    readonly "@id": string;
    readonly "@location"?: string;
};
declare namespace DataPreview {
    type Record = Pick<Data.Record, "@t" | "@name">;
    type Error = Pick<Data.Error, "@t" | "@stack">;
    type RegExp = Pick<Data.RegExp, "@t" | "@name">;
    type Collection = Pick<Data.Collection, "@t" | "@name" | "@size">;
    type Array = Pick<Data.Array, "@t" | "@size" | "@name">;
    type Function = Pick<Data.Function, "@t">;
    type Element = Pick<Data.Element, "@t" | "@name">;
    type Promise = Pick<Data.Promise, "@t">;
    type Date = Pick<Data.Date, "@t" | "@value">;
    type TypedArray = Pick<Data.TypedArray, "@t" | "@size" | "@name">;
    type Buffer = Pick<Data.Buffer, "@t" | "@size" | "@name">;
    type DataView = Pick<Data.DataView, "@t" | "@size" | "@name">;
    interface objReal {
        readonly [name: string]: RealItem<Record | Error | RegExp | Collection | Array | Function | Element | Promise | Date | TypedArray | Buffer | DataView | Data.String | Data.Number | Data.BigInt | Data.Symbol | Data.Nill>;
    }
}
declare function createPreviewValue(value: unknown): DataPreview.objReal[""]["@value"];

export { Data, DataPreview, Encode, _Encode, _getListLink, callFnLink, clearLinkStore, createPreviewValue, readLinkObject };
